package v1

import (
	"context"
	"encoding/json"
	"go.uber.org/zap"
	"time"
	"tower-monitor/consts"
	"tower-monitor/model"
	"tower-monitor/pb"
	"tower-monitor/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
)

type TowerAlarmData struct {
	DateTime  int64   `json:"date_time"`
	High      float64 `json:"high"`
	Range     float64 `json:"range"`
	Backturn  float64 `json:"backturn"`
	Weight    int64   `json:"weight"`
	WindSpeed float64 `json:"wind_speed"`
	Tilt      float64 `json:"tilt"`

	WeightPercent    int64  `json:"weight_percent"`
	MomentPercent    int64  `json:"moment_percent"`
	WindSpeedPercent int64  `json:"wind_speed_percent"`
	TiltPercent      int64  `json:"tilt_percent"`
	AlarmReason      int64   `json:"alarm_reason"`
	Braking          int64   `json:"braking"`
	Sn               string `json:"sn"`
	ProjectId        int64  `json:"project_id"`
	TowerNo          int64  `json:"tower_no"`
}

func TowerAlarmDataWrite(req *TowerAlarmData) error {
	tags := map[string]string{"sn": req.Sn}
	fields := map[string]interface{}{}

	bytes, _ := json.Marshal(req)
	json.Unmarshal(bytes, &fields)
	delete(fields, "sn")
	t := time.Now()

	// 写数据
	err := model.WriteTowerData(req.Sn, tags, fields, t, "alarm_data")
	if err != nil {
		return err
	}
	return nil
}

const (
	TowerAlarmCollide          = "T01"
	TowerAlarmWeight           = "T02"
	TowerAlarmMoment           = "T03"
	TowerAlarmTilt             = "T04"
	TowerAlarmWind             = "T05"
	TowerAlarmRangePosition    = "T06"
	TowerAlarmHighPosition     = "T07"
	TowerAlarmBackturnPosition = "T08"

	TowerWarningCollide          = "T09"
	TowerWarningWeight           = "T10"
	TowerWarningMoment           = "T11"
	TowerWarningTilt             = "T12"
	TowerWarningWind             = "T13"
	TowerWarningRangePosition    = "T14"
	TowerWarningHighPosition     = "T15"
	TowerWarningBackturnPosition = "T16"
)

var TowerAlarmValueCodeMap = map[int64]string{
	111: TowerAlarmCollide,
	12:  TowerAlarmWeight,
	13:  TowerAlarmMoment,
	14:  TowerAlarmTilt,
	15:  TowerAlarmWind,
	101: TowerAlarmRangePosition,
	102: TowerAlarmHighPosition,
	103: TowerAlarmBackturnPosition,

	211: TowerWarningCollide,
	22:  TowerWarningWeight,
	23:  TowerWarningMoment,
	24:  TowerWarningTilt,
	25:  TowerWarningWind,
	201: TowerWarningRangePosition,
	202: TowerWarningHighPosition,
	203: TowerWarningBackturnPosition,
}

var TowerAlarmValueReasonMap = map[int64]string{
	111: "碰撞报警",
	12:  "重量报警",
	13:  "力矩报警",
	14:  "倾斜报警",
	15:  "风速报警",
	101: "幅度限位报警",
	102: "高度限位报警",
	103: "回转限位报警",

	211: "碰撞预警",
	22:  "重量预警",
	23:  "力矩预警",
	24:  "倾斜预警",
	25:  "风速预警",
	201: "幅度限位预警",
	202: "高度限位预警",
	203: "回转限位预警",
}

func handleAlarm(alarmValue int64, sn string, towerno int64, timestamp int64, projectId int64, deviceName string)  {

	code := TowerAlarmValueCodeMap[alarmValue]
	if code == "" {
		return
	}
	timeStr := time.Unix(timestamp, 0).Format("2006-01-02 15:04:05")
	//reason := fmt.Sprintf("%d号%s", sn, towerno, timeStr, TowerAlarmValueReasonMap[alarmValue])

	alarmData := &v1.AlarmAddRequest{
		AlarmReason:TowerAlarmValueReasonMap[alarmValue],
		AlarmCode:code,
		Date:timeStr,
		Sn:sn,
		ProjectId:projectId,
		DeviceCode:consts.TowerDeviceCode,
		DeviceType:consts.TowerDeviceName,
		DeviceName:deviceName,

	}

	// 添加告警
	_, err := pb.SmartAlarm.AlarmAdd(context.Background(), alarmData)
	if err != nil {
		reqByte, _ := json.Marshal(alarmData)
		logger.Error("rpc",
			zap.String("call", "pb.SmartAlarm.AlarmAdd"),
			zap.String("args", string(reqByte)),
			zap.String("error", err.Error()))
	}

}

func TowerFrameAlarmRequestHandle(sn string, towerNo byte, version byte, data []byte) (res []byte, err error) {
	// 不管，因为实时包里包含了报警数据
	return makeResponse(sn, towerNo, TowerFrameAlarmResponse, version, []byte{0x00}), nil

	projectId, devicename := checkAuth(sn)
	if projectId <= 0 {
		// 未认证或需重新认证
		return makeResponse(sn, towerNo, TowerFrameAlarmResponse, version, []byte{0xfe}), nil
	}
	if len(data) < 22 {
		return makeResponse(sn, towerNo, TowerFrameAlarmResponse, version, []byte{0xff}), nil
	}

	record := &TowerAlarmData{}
	record.Sn = sn
	record.TowerNo = int64(towerNo)
	record.ProjectId = projectId

	// 时间戳
	record.DateTime = int64(data[0])<<24 + int64(data[1])<<16 + int64(data[2])<<8 + int64(data[3])

	// 警报原因
	// 211 212 213 214 碰撞预警
	// 111 112 113 114 碰撞报警
	// 12 13 14 15 重量 力矩 倾斜 风速报警
	// 22 23 24 25 重量 力矩 倾斜 风速预警
	record.AlarmReason = int64(data[4])

	// 高度
	record.High = (float64(int16(data[5])<<8) + float64(data[6])) * 0.1

	// 幅度
	record.Range = (float64(int16(data[7])<<8) + float64(data[8])) * 0.1

	// 回转
	record.Backturn = (float64(int16(data[9])<<8) + float64(data[10])) * 0.1

	// 起重量数据
	record.Weight = int64(data[11])<<8 + int64(data[12])

	// 风速
	record.WindSpeed = (float64(int16(data[13])<<8) + float64(data[14])) * 0.1

	// 倾角
	record.Tilt = (float64(int16(data[15])<<8) + float64(data[16])) * 0.1

	// 重量百分比
	record.WeightPercent = int64(data[17])

	// 力矩百分比
	record.MomentPercent = int64(data[18])

	// 风速百分比
	record.WindSpeedPercent = int64(data[19])

	// 倾斜百分比
	record.TiltPercent = int64(data[20])

	// 制动状态：1个字节，8bit 从低到高分别是 上下前后左右
	// Bit 1：表示断开； bit 0：表示闭合
	record.Braking = int64(data[21])

	if TowerAlarmDataWrite(record) != nil {
		return makeResponse(sn, towerNo, TowerFrameAlarmResponse, version, []byte{0xfd}), nil
	}
	handleAlarm(record.AlarmReason, sn, int64(towerNo), record.DateTime, projectId, devicename)

	return makeResponse(sn, towerNo, TowerFrameAlarmResponse, version, []byte{0x00}), nil
}
